@@ -21,9 +21,17 @@ class AudioMaterial(Enum):
     ceramic = 0
     glass = 1
     metal = 2
-    hardwood = 3
-    wood = 4
-    cardboard = 5
+    wood_hard = 3
+    wood_medium = 4
+    wood_soft = 5
+    cardboard = 6
+    paper = 7
+    plastic_hard = 8
+    plastic_soft_foam = 9
+    rubber = 10
+    fabric = 11
+    leather = 12
+    stone = 13
 
 
 class ObjectInfo:
@@ -32,7 +40,7 @@ class ObjectInfo:
     The audio values here are just recommendations; you can apply different values if you want.
     """
 
-    def __init__(self, name: str, amp: float, mass: float, material: AudioMaterial, library: str, bounciness: float, resonance: float):
+    def __init__(self, name: str, amp: float, mass: float, material: AudioMaterial, library: str, bounciness: float, resonance: float, size: int):
         """
         :param name: The model name.
         :param amp: The sound amplitude.
@@ -41,6 +49,7 @@ class ObjectInfo:
         :param library: The path to the model library (see ModelLibrarian documentation).
         :param bounciness: The bounciness value for a Unity physics material.
         :param resonance: The resonance value for the object.
+        :param size: Integer representing the size "bucket" this object belongs to (0-5).
         """
 
         self.amp = amp
@@ -50,14 +59,23 @@ class ObjectInfo:
         self.name = name
         self.bounciness = bounciness
         self.resonance = resonance
+        self.size = size
 
 
 # Density per audio material.
 DENSITIES: Dict[AudioMaterial, float] = {AudioMaterial.ceramic: 2180,
                                          AudioMaterial.glass: 2500,
+                                         AudioMaterial.stone: 2000,
                                          AudioMaterial.metal: 8450,
-                                         AudioMaterial.hardwood: 900,
-                                         AudioMaterial.wood: 690,
+                                         AudioMaterial.wood_hard: 1200,
+                                         AudioMaterial.wood_medium: 700,
+                                         AudioMaterial.wood_soft: 400,
+                                         AudioMaterial.fabric: 1540,
+                                         AudioMaterial.leather: 860,
+                                         AudioMaterial.plastic_hard: 1150,
+                                         AudioMaterial.plastic_soft_foam: 285,
+                                         AudioMaterial.rubber: 1522,
+                                         AudioMaterial.paper: 1200,
                                          AudioMaterial.cardboard: 698}
 
 
@@ -326,11 +344,14 @@ class PyImpact:
 
         # Cache the material data. This is use to reset the material modes.
         self.material_data: Dict[str, dict] = {}
-        for mat, path in zip(["ceramic", "hardwood", "metal", "glass", "wood", "cardboard"],
-                             ["Ceramic_mm", "Poplar_mm", "MetalStrip_mm", "Mirror_mm", "BalsaWood_mm", "Cardboard_mm"]):
-            # Load the JSON data.
-            data = json.loads(Path(resource_filename(__name__, f"py_impact/material_data/{path}.json")).read_text())
-            self.material_data.update({mat: data})
+        material_list = ["ceramic", "wood_hard", "wood_medium", "wood_soft", "metal", "glass", "paper", "cardboard", "leather", "fabric", "plastic_hard", "plastic_soft_foam", "rubber", "stone"]
+        for mat in material_list:
+            for i in range(6):
+                # Load the JSON data.
+                mat_name = mat + "_" + str(i)
+                path = mat_name + "_mm"
+                data = json.loads(Path(resource_filename(__name__, f"py_impact/material_data/{path}.json")).read_text())
+                self.material_data.update({mat_name: data})
 
         # Create empty dictionary for log.
         self.mode_properties_log = dict()
@@ -414,10 +435,10 @@ class PyImpact:
                                                           rigidbodies=rigidbodies,
                                                           target_id=target,
                                                           target_amp=target_audio.amp,
-                                                          target_mat=target_audio.material.name,
+                                                          target_mat=target_audio.material.name + "_" + str(target_audio.size),
                                                           other_id=other,
                                                           other_amp=other_audio.amp,
-                                                          other_mat=other_audio.material.name,
+                                                          other_mat=other_audio.material.name + "_" + str(other_audio.size),
                                                           resonance=target_audio.resonance,
                                                           play_audio_data=not resonance_audio))
         # Play sounds from collisions with the environment.
@@ -437,10 +458,12 @@ class PyImpact:
                                                           rigidbodies=rigidbodies,
                                                           target_id=target,
                                                           target_amp=audio.amp,
-                                                          target_mat=audio.material.name,
+                                                          target_mat=audio.material.name + "_" + str(audio.size),
                                                           other_id=self.env_id,
                                                           other_amp=0.5,
-                                                          other_mat=floor.name,
+                                                          # We probably need dedicated wall and floor materials, or maybe they are in size category #6?
+                                                          # Setting to "4" for now, for general debugging purposes
+                                                          other_mat=floor.name + "_4",
                                                           resonance=audio.resonance,
                                                           play_audio_data=not resonance_audio))
         return commands
@@ -458,7 +481,6 @@ class PyImpact:
 
         :return: The audio modes.
         """
-
         data = self.material_data[material] if isinstance(material, str) else self.material_data[material.name]
         # Load the mode properties.
         f = -1
@@ -752,7 +774,7 @@ class PyImpact:
             for row in reader:
                 o = ObjectInfo(name=row["name"], amp=float(row["amp"]), mass=float(row["mass"]),
                                material=AudioMaterial[row["material"]], library=row["library"],
-                               bounciness=float(row["bounciness"]), resonance=float(row["resonance"]))
+                               bounciness=float(row["bounciness"]), resonance=float(row["resonance"]), size=int(row["size"]))
                 objects.update({o.name: o})
 
         return objects
